@@ -105,20 +105,18 @@ matd_t *matd_identity(int dim)
 }
 
 // row and col are zero-based
-TYPE matd_get(const matd_t *m, int row, int col)
+TYPE matd_get(const matd_t *m, unsigned int row, unsigned int col)
 {
     assert(m != NULL);
     assert(!matd_is_scalar(m));
-    assert(row >= 0);
     assert(row < m->nrows);
-    assert(col >= 0);
     assert(col < m->ncols);
 
     return MATD_EL(m, row, col);
 }
 
 // row and col are zero-based
-void matd_put(matd_t *m, int row, int col, TYPE value)
+void matd_put(matd_t *m, unsigned int row, unsigned int col, TYPE value)
 {
     assert(m != NULL);
 
@@ -127,9 +125,7 @@ void matd_put(matd_t *m, int row, int col, TYPE value)
         return;
     }
 
-    assert(row >= 0);
     assert(row < m->nrows);
-    assert(col >= 0);
     assert(col < m->ncols);
 
     MATD_EL(m, row, col) = value;
@@ -164,12 +160,12 @@ matd_t *matd_copy(const matd_t *m)
     return x;
 }
 
-matd_t *matd_select(const matd_t * a, int r0, int r1, int c0, int c1)
+matd_t *matd_select(const matd_t * a, unsigned int r0, int r1, unsigned int c0, int c1)
 {
     assert(a != NULL);
 
-    assert(r0 >= 0 && r0 < a->nrows);
-    assert(c0 >= 0 && c0 < a->ncols);
+    assert(r0 < a->nrows);
+    assert(c0 < a->ncols);
 
     int nrows = r1 - r0 + 1;
     int ncols = c1 - c0 + 1;
@@ -192,8 +188,8 @@ void matd_print(const matd_t *m, const char *fmt)
         printf(fmt, MATD_EL(m, 0, 0));
         printf("\n");
     } else {
-        for (int i = 0; i < m->nrows; i++) {
-            for (int j = 0; j < m->ncols; j++) {
+        for (unsigned int i = 0; i < m->nrows; i++) {
+            for (unsigned int j = 0; j < m->ncols; j++) {
                 printf(fmt, MATD_EL(m, i, j));
             }
             printf("\n");
@@ -210,8 +206,8 @@ void matd_print_transpose(const matd_t *m, const char *fmt)
         printf(fmt, MATD_EL(m, 0, 0));
         printf("\n");
     } else {
-        for (int j = 0; j < m->ncols; j++) {
-            for (int i = 0; i < m->nrows; i++) {
+        for (unsigned int j = 0; j < m->ncols; j++) {
+            for (unsigned int i = 0; i < m->nrows; i++) {
                 printf(fmt, MATD_EL(m, i, j));
             }
             printf("\n");
@@ -241,10 +237,10 @@ matd_t *matd_multiply(const matd_t *a, const matd_t *b)
     assert(a->ncols == b->nrows);
     matd_t *m = matd_create(a->nrows, b->ncols);
 
-    for (int i = 0; i < m->nrows; i++) {
-        for (int j = 0; j < m->ncols; j++) {
+    for (unsigned int i = 0; i < m->nrows; i++) {
+        for (unsigned int j = 0; j < m->ncols; j++) {
             TYPE acc = 0;
-            for (int k = 0; k < a->ncols; k++) {
+            for (unsigned int k = 0; k < a->ncols; k++) {
                 acc += MATD_EL(a, i, k) * MATD_EL(b, k, j);
             }
             MATD_EL(m, i, j) = acc;
@@ -263,8 +259,8 @@ matd_t *matd_scale(const matd_t *a, double s)
 
     matd_t *m = matd_create(a->nrows, a->ncols);
 
-    for (int i = 0; i < m->nrows; i++) {
-        for (int j = 0; j < m->ncols; j++) {
+    for (unsigned int i = 0; i < m->nrows; i++) {
+        for (unsigned int j = 0; j < m->ncols; j++) {
             MATD_EL(m, i, j) = s * MATD_EL(a, i, j);
         }
     }
@@ -281,8 +277,8 @@ void matd_scale_inplace(matd_t *a, double s)
         return;
     }
 
-    for (int i = 0; i < a->nrows; i++) {
-        for (int j = 0; j < a->ncols; j++) {
+    for (unsigned int i = 0; i < a->nrows; i++) {
+        for (unsigned int j = 0; j < a->ncols; j++) {
             MATD_EL(a, i, j) *= s;
         }
     }
@@ -300,8 +296,8 @@ matd_t *matd_add(const matd_t *a, const matd_t *b)
 
     matd_t *m = matd_create(a->nrows, a->ncols);
 
-    for (int i = 0; i < m->nrows; i++) {
-        for (int j = 0; j < m->ncols; j++) {
+    for (unsigned int i = 0; i < m->nrows; i++) {
+        for (unsigned int j = 0; j < m->ncols; j++) {
             MATD_EL(m, i, j) = MATD_EL(a, i, j) + MATD_EL(b, i, j);
         }
     }
@@ -321,8 +317,8 @@ void matd_add_inplace(matd_t *a, const matd_t *b)
         return;
     }
 
-    for (int i = 0; i < a->nrows; i++) {
-        for (int j = 0; j < a->ncols; j++) {
+    for (unsigned int i = 0; i < a->nrows; i++) {
+        for (unsigned int j = 0; j < a->ncols; j++) {
             MATD_EL(a, i, j) += MATD_EL(b, i, j);
         }
     }
@@ -341,8 +337,8 @@ matd_t *matd_subtract(const matd_t *a, const matd_t *b)
 
     matd_t *m = matd_create(a->nrows, a->ncols);
 
-    for (int i = 0; i < m->nrows; i++) {
-        for (int j = 0; j < m->ncols; j++) {
+    for (unsigned int i = 0; i < m->nrows; i++) {
+        for (unsigned int j = 0; j < m->ncols; j++) {
             MATD_EL(m, i, j) = MATD_EL(a, i, j) - MATD_EL(b, i, j);
         }
     }
@@ -362,8 +358,8 @@ void matd_subtract_inplace(matd_t *a, const matd_t *b)
         return;
     }
 
-    for (int i = 0; i < a->nrows; i++) {
-        for (int j = 0; j < a->ncols; j++) {
+    for (unsigned int i = 0; i < a->nrows; i++) {
+        for (unsigned int j = 0; j < a->ncols; j++) {
             MATD_EL(a, i, j) -= MATD_EL(b, i, j);
         }
     }
@@ -379,8 +375,8 @@ matd_t *matd_transpose(const matd_t *a)
 
     matd_t *m = matd_create(a->ncols, a->nrows);
 
-    for (int i = 0; i < a->nrows; i++) {
-        for (int j = 0; j < a->ncols; j++) {
+    for (unsigned int i = 0; i < a->nrows; i++) {
+        for (unsigned int j = 0; j < a->ncols; j++) {
             MATD_EL(m, j, i) = MATD_EL(a, i, j);
         }
     }
@@ -398,7 +394,7 @@ double matd_det_general(const matd_t *a)
     // The determinants of the L and U matrices are the products of
     // their respective diagonal elements
     double detL = 1; double detU = 1;
-    for (int i = 0; i < a->nrows; i++) {
+    for (unsigned int i = 0; i < a->nrows; i++) {
         detL *= matd_get(L, i, i);
         detU *= matd_get(U, i, i);
     }
@@ -876,6 +872,8 @@ double matd_vec_dist_n(const matd_t *a, const matd_t *b, int n)
     int lenb = b->nrows*b->ncols;
 
     assert(n <= lena && n <= lenb);
+    (void)lena;
+    (void)lenb;
 
     double mag = 0.0;
     for (int i = 0; i < n; i++)
@@ -910,6 +908,7 @@ double matd_vec_dot_product(const matd_t *a, const matd_t *b)
     int adim = a->ncols*a->nrows;
     int bdim = b->ncols*b->nrows;
     assert(adim == bdim);
+    (void)bdim;
 
     double acc = 0;
     for (int i = 0; i < adim; i++) {
@@ -958,8 +957,8 @@ TYPE matd_err_inf(const matd_t *a, const matd_t *b)
 
     TYPE maxf = 0;
 
-    for (int i = 0; i < a->nrows; i++) {
-        for (int j = 0; j < a->ncols; j++) {
+    for (unsigned int i = 0; i < a->nrows; i++) {
+        for (unsigned int j = 0; j < a->ncols; j++) {
             TYPE av = MATD_EL(a, i, j);
             TYPE bv = MATD_EL(b, i, j);
 
@@ -1003,7 +1002,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
     // RS: cumulative right-handed transformations.
     matd_t *RS = matd_identity(A->ncols);
 
-    for (int hhidx = 0; hhidx < A->nrows; hhidx++)  {
+    for (unsigned int hhidx = 0; hhidx < A->nrows; hhidx++)  {
 
         if (hhidx < A->ncols) {
             // We construct the normal of the reflection plane: let u
@@ -1063,7 +1062,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
             // LS = matd_op("F*M", LS, Q);
             // Implementation: take each row of LS, compute dot product with n,
             // subtract n (scaled by dot product) from it.
-            for (int i = 0; i < LS->nrows; i++) {
+            for (unsigned int i = 0; i < LS->nrows; i++) {
                 double dot = 0;
                 for (int j = 0; j < vlen; j++)
                     dot += MATD_EL(LS, i, hhidx+j) * v[j];
@@ -1072,7 +1071,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
             }
 
             //  B = matd_op("M*F", Q, B); // should be Q', but Q is symmetric.
-            for (int i = 0; i < B->ncols; i++) {
+            for (unsigned int i = 0; i < B->ncols; i++) {
                 double dot = 0;
                 for (int j = 0; j < vlen; j++)
                     dot += MATD_EL(B, hhidx+j, i) * v[j];
@@ -1121,7 +1120,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
             //       MATD_EL(Q, i+1+hhidx, j+1+hhidx) -= 2*v[i]*v[j];
 
             //  RS = matd_op("F*M", RS, Q);
-            for (int i = 0; i < RS->nrows; i++) {
+            for (unsigned int i = 0; i < RS->nrows; i++) {
                 double dot = 0;
                 for (int j = 0; j < vlen; j++)
                     dot += MATD_EL(RS, i, hhidx+1+j) * v[j];
@@ -1130,7 +1129,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
             }
 
             //   B = matd_op("F*M", B, Q); // should be Q', but Q is symmetric.
-            for (int i = 0; i < B->nrows; i++) {
+            for (unsigned int i = 0; i < B->nrows; i++) {
                 double dot = 0;
                 for (int j = 0; j < vlen; j++)
                     dot += MATD_EL(B, i, hhidx+1+j) * v[j];
@@ -1149,7 +1148,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
     assert(maxiters > 0); // reassure clang
     int iter;
 
-    double maxv; // maximum non-zero value being reduced this iteration
+    double maxv = 0; // maximum non-zero value being reduced this iteration
 
     double tol = 1E-10;
 
@@ -1159,11 +1158,11 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
 
     // for each of the first B->ncols rows, which index has the
     // maximum absolute value? (used by method 1)
-    int *maxrowidx = malloc(sizeof(int)*B->ncols);
-    int lastmaxi, lastmaxj;
+    unsigned int *maxrowidx = malloc(sizeof(int)*B->ncols);
+    unsigned int lastmaxi, lastmaxj;
 
     if (find_max_method == 1) {
-        for (int i = 2; i < B->ncols; i++)
+        for (unsigned int i = 2; i < B->ncols; i++)
             maxrowidx[i] = max_idx(B, i, B->ncols);
 
         // note that we started the array at 2. That's because by setting
@@ -1196,7 +1195,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
             // modified. Update maxrowidx accordingly.
 
             // now, EVERY row also had columns lastmaxi and lastmaxj modified.
-            for (int rowi = 0; rowi < B->ncols; rowi++) {
+            for (unsigned int rowi = 0; rowi < B->ncols; rowi++) {
 
                 // the magnitude of the largest off-diagonal element
                 // in this row.
@@ -1269,8 +1268,8 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
             maxv = -1;
 
             // only search top "square" portion
-            for (int i = 0; i < B->ncols; i++) {
-                for (int j = 0; j < B->ncols; j++) {
+            for (unsigned int i = 0; i < B->ncols; i++) {
+                for (unsigned int j = 0; j < B->ncols; j++) {
                     if (i == j)
                         continue;
 
@@ -1335,7 +1334,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
 */
 
             //  LS = matd_op("F*M", LS, QL);
-            for (int i = 0; i < LS->nrows; i++) {
+            for (unsigned int i = 0; i < LS->nrows; i++) {
                 double vi = MATD_EL(LS, i, maxi);
                 double vj = MATD_EL(LS, i, maxj);
 
@@ -1344,7 +1343,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
             }
 
             //  RS = matd_op("F*M", RS, QR); // remember we'll transpose RS.
-            for (int i = 0; i < RS->nrows; i++) {
+            for (unsigned int i = 0; i < RS->nrows; i++) {
                 double vi = MATD_EL(RS, i, maxi);
                 double vj = MATD_EL(RS, i, maxj);
 
@@ -1354,7 +1353,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
 
             // B = matd_op("M'*F*M", QL, B, QR);
             // The QL matrix mixes rows of B.
-            for (int i = 0; i < B->ncols; i++) {
+            for (unsigned int i = 0; i < B->ncols; i++) {
                 double vi = MATD_EL(B, maxi, i);
                 double vj = MATD_EL(B, maxj, i);
 
@@ -1363,7 +1362,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
             }
 
             // The QR matrix mixes columns of B.
-            for (int i = 0; i < B->nrows; i++) {
+            for (unsigned int i = 0; i < B->nrows; i++) {
                 double vi = MATD_EL(B, i, maxi);
                 double vj = MATD_EL(B, i, maxj);
 
@@ -1386,7 +1385,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
     // U/LS.
     int *idxs = malloc(sizeof(int)*A->ncols);
     double *vals = malloc(sizeof(double)*A->ncols);
-    for (int i = 0; i < A->ncols; i++) {
+    for (unsigned int i = 0; i < A->ncols; i++) {
         idxs[i] = i;
         vals[i] = MATD_EL(B, i, i);
     }
@@ -1396,7 +1395,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
     do {
         changed = 0;
 
-        for (int i = 0; i + 1 < A->ncols; i++) {
+        for (unsigned int i = 0; i + 1 < A->ncols; i++) {
             if (fabs(vals[i+1]) > fabs(vals[i])) {
                 int tmpi = idxs[i];
                 idxs[i] = idxs[i+1];
@@ -1414,7 +1413,7 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
     matd_t *LP = matd_identity(A->nrows);
     matd_t *RP = matd_identity(A->ncols);
 
-    for (int i = 0; i < A->ncols; i++) {
+    for (unsigned int i = 0; i < A->ncols; i++) {
         MATD_EL(LP, idxs[i], idxs[i]) = 0; // undo the identity above
         MATD_EL(RP, idxs[i], idxs[i]) = 0;
 
@@ -1442,8 +1441,8 @@ static matd_svd_t matd_svd_tall(matd_t *A, int flags)
 
     // make B exactly diagonal
 
-    for (int i = 0; i < B->nrows; i++) {
-        for (int j = 0; j < B->ncols; j++) {
+    for (unsigned int i = 0; i < B->nrows; i++) {
+        for (unsigned int j = 0; j < B->ncols; j++) {
             if (i != j)
                 MATD_EL(B, i, j) = 0;
         }
@@ -1520,11 +1519,11 @@ matd_plu_t *matd_plu(const matd_t *a)
 
     matd_plu_t *mlu = calloc(1, sizeof(matd_plu_t));
 
-    for (int i = 0; i < a->nrows; i++)
+    for (unsigned int i = 0; i < a->nrows; i++)
         piv[i] = i;
 
-    for (int j = 0; j < a->ncols; j++) {
-        for (int i = 0; i < a->nrows; i++) {
+    for (unsigned int j = 0; j < a->ncols; j++) {
+        for (unsigned int i = 0; i < a->nrows; i++) {
             int kmax = i < j ? i : j; // min(i,j)
 
             // compute dot product of row i with column j (up through element kmax)
@@ -1536,9 +1535,9 @@ matd_plu_t *matd_plu(const matd_t *a)
         }
 
         // find pivot and exchange if necessary.
-        int p = j;
+        unsigned int p = j;
         if (1) {
-            for (int i = j+1; i < lu->nrows; i++) {
+            for (unsigned int i = j+1; i < lu->nrows; i++) {
                 if (fabs(MATD_EL(lu,i,j)) > fabs(MATD_EL(lu, p, j))) {
                     p = i;
                 }
@@ -1577,7 +1576,7 @@ matd_plu_t *matd_plu(const matd_t *a)
 
         if (j < lu->ncols && j < lu->nrows && LUjj != 0) {
             LUjj = 1.0 / LUjj;
-            for (int i = j+1; i < lu->nrows; i++)
+            for (unsigned int i = j+1; i < lu->nrows; i++)
                 MATD_EL(lu, i, j) *= LUjj;
         }
     }
@@ -1603,7 +1602,7 @@ double matd_plu_det(const matd_plu_t *mlu)
     double det = mlu->pivsign;
 
     if (lu->nrows == lu->ncols) {
-        for (int i = 0; i < lu->ncols; i++)
+        for (unsigned int i = 0; i < lu->ncols; i++)
             det *= MATD_EL(lu, i, i);
     }
 
@@ -1615,7 +1614,7 @@ matd_t *matd_plu_p(const matd_plu_t *mlu)
     matd_t *lu = mlu->lu;
     matd_t *P = matd_create(lu->nrows, lu->nrows);
 
-    for (int i = 0; i < lu->nrows; i++) {
+    for (unsigned int i = 0; i < lu->nrows; i++) {
         MATD_EL(P, mlu->piv[i], i) = 1;
     }
 
@@ -1627,10 +1626,10 @@ matd_t *matd_plu_l(const matd_plu_t *mlu)
     matd_t *lu = mlu->lu;
 
     matd_t *L = matd_create(lu->nrows, lu->ncols);
-    for (int i = 0; i < lu->nrows; i++) {
+    for (unsigned int i = 0; i < lu->nrows; i++) {
         MATD_EL(L, i, i) = 1;
 
-        for (int j = 0; j < i; j++) {
+        for (unsigned int j = 0; j < i; j++) {
             MATD_EL(L, i, j) = MATD_EL(lu, i, j);
         }
     }
@@ -1643,8 +1642,8 @@ matd_t *matd_plu_u(const matd_plu_t *mlu)
     matd_t *lu = mlu->lu;
 
     matd_t *U = matd_create(lu->ncols, lu->ncols);
-    for (int i = 0; i < lu->ncols; i++) {
-        for (int j = 0; j < lu->ncols; j++) {
+    for (unsigned int i = 0; i < lu->ncols; i++) {
+        for (unsigned int j = 0; j < lu->ncols; j++) {
             if (i <= j)
                 MATD_EL(U, i, j) = MATD_EL(lu, i, j);
         }
@@ -1662,14 +1661,14 @@ matd_t *matd_plu_solve(const matd_plu_t *mlu, const matd_t *b)
     matd_t *x = matd_copy(b);
 
     // permute right hand side
-    for (int i = 0; i < mlu->lu->nrows; i++)
+    for (unsigned int i = 0; i < mlu->lu->nrows; i++)
         memcpy(&MATD_EL(x, i, 0), &MATD_EL(b, mlu->piv[i], 0), sizeof(TYPE) * b->ncols);
 
     // solve Ly = b
-    for (int k = 0; k < mlu->lu->nrows; k++) {
-        for (int i = k+1; i < mlu->lu->nrows; i++) {
+    for (unsigned int k = 0; k < mlu->lu->nrows; k++) {
+        for (unsigned int i = k+1; i < mlu->lu->nrows; i++) {
             double LUik = -MATD_EL(mlu->lu, i, k);
-            for (int t = 0; t < b->ncols; t++)
+            for (unsigned int t = 0; t < b->ncols; t++)
                 MATD_EL(x, i, t) += MATD_EL(x, k, t) * LUik;
         }
     }
@@ -1677,12 +1676,12 @@ matd_t *matd_plu_solve(const matd_plu_t *mlu, const matd_t *b)
     // solve Ux = y
     for (int k = mlu->lu->ncols-1; k >= 0; k--) {
         double LUkk = 1.0 / MATD_EL(mlu->lu, k, k);
-        for (int t = 0; t < b->ncols; t++)
+        for (unsigned int t = 0; t < b->ncols; t++)
             MATD_EL(x, k, t) *= LUkk;
 
         for (int i = 0; i < k; i++) {
             double LUik = -MATD_EL(mlu->lu, i, k);
-            for (int t = 0; t < b->ncols; t++)
+            for (unsigned int t = 0; t < b->ncols; t++)
                 MATD_EL(x, i, t) += MATD_EL(x, k, t) *LUik;
         }
     }
@@ -1910,7 +1909,7 @@ void matd_ltransposetriangle_solve(matd_t *u, const TYPE *b, TYPE *x)
     for (int i = 0; i < n; i++) {
         x[i] /= MATD_EL(u, i, i);
 
-        for (int j = i+1; j < u->ncols; j++) {
+        for (unsigned int j = i+1; j < u->ncols; j++) {
             x[j] -= x[i] * MATD_EL(u, i, j);
         }
     }
@@ -1940,7 +1939,7 @@ void matd_utriangle_solve(matd_t *u, const TYPE *b, TYPE *x)
 
         double diag = MATD_EL(u, i, i);
 
-        for (int j = i+1; j < u->ncols; j++)
+        for (unsigned int j = i+1; j < u->ncols; j++)
             bi -= MATD_EL(u, i, j)*x[j];
 
         x[i] = bi / diag;
@@ -1957,17 +1956,17 @@ matd_t *matd_chol_solve(const matd_chol_t *chol, const matd_t *b)
 
     // solve Ly = b ==> (U')y = b
 
-    for (int i = 0; i < u->nrows; i++) {
-        for (int j = 0; j < i; j++) {
+    for (unsigned int i = 0; i < u->nrows; i++) {
+        for (unsigned int j = 0; j < i; j++) {
             // b[i] -= L[i,j]*x[j]... replicated across columns of b
             //   ==> i.e., ==>
             // b[i,k] -= L[i,j]*x[j,k]
-            for (int k = 0; k < b->ncols; k++) {
+            for (unsigned int k = 0; k < b->ncols; k++) {
                 MATD_EL(x, i, k) -= MATD_EL(u, j, i)*MATD_EL(x, j, k);
             }
         }
         // x[i] = b[i] / L[i,i]
-        for (int k = 0; k < b->ncols; k++) {
+        for (unsigned int k = 0; k < b->ncols; k++) {
             MATD_EL(x, i, k) /= MATD_EL(u, i, i);
         }
     }
@@ -1975,12 +1974,12 @@ matd_t *matd_chol_solve(const matd_chol_t *chol, const matd_t *b)
     // solve Ux = y
     for (int k = u->ncols-1; k >= 0; k--) {
         double LUkk = 1.0 / MATD_EL(u, k, k);
-        for (int t = 0; t < b->ncols; t++)
+        for (unsigned int t = 0; t < b->ncols; t++)
             MATD_EL(x, k, t) *= LUkk;
 
         for (int i = 0; i < k; i++) {
             double LUik = -MATD_EL(u, i, k);
-            for (int t = 0; t < b->ncols; t++)
+            for (unsigned int t = 0; t < b->ncols; t++)
                 MATD_EL(x, i, t) += MATD_EL(x, k, t) *LUik;
         }
     }
@@ -2016,8 +2015,8 @@ matd_t *matd_chol_inverse(matd_t *a)
 double matd_max(matd_t *m)
 {
     double d = -DBL_MAX;
-    for(int x=0; x<m->nrows; x++) {
-        for(int y=0; y<m->ncols; y++) {
+    for(unsigned int x=0; x<m->nrows; x++) {
+        for(unsigned int y=0; y<m->ncols; y++) {
             if(MATD_EL(m, x, y) > d)
                 d = MATD_EL(m, x, y);
         }
